package jws.awt;

// Copyright 1997, John Webster Small
// All rights Reserved

import java.awt.*;
import java.awt.event.*;
import jws.awt.event.*;

public class ChoiceCardPanel extends OutlinedPanel
  implements ItemSelectable
{
  private Choice pageNames;
  private Panel pages;

  public int getSelectedIndex()
    { return pageNames.getSelectedIndex(); }

  public String getSelectedItem()
    { return pageNames.getSelectedItem(); }

  public void addItemListener(ItemListener l)
    { pageNames.addItemListener(l); }

  public Object[] getSelectedObjects()
    { return pageNames.getSelectedObjects(); }

  public void removeItemListener(ItemListener l)
    { pageNames.removeItemListener(l); }

  /** Switch to named card.
    * @see java.awt.CardLayout#show
    */
  public void select(String name)
  {
    ((CardLayout)pages.getLayout()).show(pages,name);
  }

  public void select(int pos)
  {
    if (pos < pageNames.getItemCount())
      select(pageNames.getItem(pos));
  }

  /** Construct ChoiceCardPanel. */
  public ChoiceCardPanel(String title)
  {
    super(title);
    setLayout(new BorderLayout());
    pageNames = new Choice();
    pageNames.addItemListener
      (
        new ItemAdapter()
        {
          public void itemStateChanged(ItemEvent e)
          {
            if (e.getStateChange() == ItemEvent.SELECTED)
              select((String)e.getItem());
          }
        }
      );
    super.add("North",pageNames);
    pages = new Panel();
    pages.setLayout(new CardLayout());
    super.add("Center",pages);
  }

  /** Add named component.
    * @param name name of component which appears in Choice
    * @param comp component to appear when chosen
    */
  public Component add(String name, Component comp)
  {
    pageNames.addItem(name);
    pages.add(name,comp);
    return comp;
  }

  public Component add(Component comp, int index)
  {
    if (index < pageNames.getItemCount())
      pages.add(pageNames.getItem(index),comp);
    return comp;
  }

  public Component add(Component comp)
  {
    return add(comp,0);
  }

  public int indexOf(String name)
  {
    for (int i = 0; i < pageNames.getItemCount(); i++)
      if (pageNames.getItem(i).equals(name))
        return i;
    return -1;
  }

  public void remove(String name)
  {
    remove(indexOf(name));
  }

  public Component getItem(int index)
  {
    return pages.getComponent(index);
  }

  public Component getItem(String name)
  {
    return getItem(indexOf(name));
  }

  public void remove(int index)
  {
    if (index < pageNames.getItemCount())  {
      pageNames.remove(index);
      pages.remove(index);
    }
  }

  public void removeAll()
  {
    pageNames.removeAll();
    pages.removeAll();
  }

/*
  public static void main(String[] args)
  {
    ChoiceCardPanel ccp = new ChoiceCardPanel("Folders");
    ccp.add("One",new Button("One"));
    ccp.add("TextArea",new TextArea());
    new WinFrame("ChoiceCardPanel Demo",ccp)
      .appMainWindow().setVisible(true);
  }
*/
}
